<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * The main clickmeeting configuration form
 *
 * It uses the standard core Moodle formslib. For more info about them, please
 * visit: http://docs.moodle.org/en/Development:lib/formslib.php
 *
 * @package    mod
 * @subpackage clickmeeting
 * @copyright  2011 Your Name
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot.'/course/moodleform_mod.php');

/**
 * Module instance settings form
 */
class mod_clickmeeting_mod_form extends moodleform_mod {

    /**
     * Defines forms elements
     */
    public function definition() {
        global $CFG;

        $mform = $this->_form;

        //-------------------------------------------------------------------------------
        // Adding the "general" fieldset, where all the common settings are showed
        $mform->addElement('header', 'general', get_string('general', 'form'));

        // Adding the standard "name" field
        $mform->addElement('text', 'name', get_string('clickmeetingname', 'clickmeeting'), array('size'=>'64'));
        if (!empty($CFG->formatstringstriptags)) {
            $mform->setType('name', PARAM_TEXT);
        } else {
            $mform->setType('name', PARAM_CLEAN);
        }
        $mform->addRule('name', null, 'required', null, 'client');
        $mform->addRule('name', get_string('maximumchars', '', 255), 'maxlength', 255, 'client');
        $mform->addHelpButton('name', 'clickmeetingname', 'clickmeeting');

        $mform->addElement('textarea', 'description', get_string('description'), array('rows'  => 10, 'cols'  => 62));
//        $mform->setType('description', PARAM_RAW);


        // Adding the textarea to waiting message
        $mform->addElement('textarea', 'lobby_msg', get_string("form:waitingmsg", "clickmeeting"), 'rows="10" cols="62"');
        $mform->addRule('lobby_msg', get_string('maximumchars', '', 255), 'maxlength', 255, 'client');
        $mform->addHelpButton('lobby_msg', 'form:waitingmsg', 'clickmeeting');


        $mform->addElement('header', 'clickmeetingfieldset', get_string('form:data_header', 'clickmeeting'));
        $mform->addElement('date_time_selector', 'timestart', get_string('from'));
//        $mform->addElement('date_time_selector', 'assesstimestart', get_string('to'));
//        $mform->addElement('duration', 'duration', get_string('timelimit', 'quiz'));

        $mform->addElement('select', 'duration', get_string('timelimit', 'quiz'), array('1' => 1, '2' => 2, '3' => 3));
        $mform->addHelpButton('duration', 'form:maxduration', 'clickmeeting');

        //-------------------------------------------------------------------------------
        // add standard elements, common to all modules
        $this->standard_coursemodule_elements();
        //-------------------------------------------------------------------------------
        // add standard buttons, common to all modules
        $this->add_action_buttons();
    }
}
