<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Prints a particular instance of clickmeeting
 *
 * You can have a rather longer description of the file as well,
 * if you like, and it can span multiple lines.
 *
 * @package    mod
 * @subpackage clickmeeting
 * @copyright  2013  Łukasz Wojciechowski
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once(dirname(dirname(dirname(__FILE__))).'/config.php');
require_once(dirname(__FILE__).'/lib.php');

$id = optional_param('id', 0, PARAM_INT); // course_module ID, or
$n  = optional_param('n', 0, PARAM_INT);  // clickmeeting instance ID - it should be named as the first character of the module

global $COURSE, $USER, $THEME;

// Add the required JavaScript to the page
$THEME = new stdClass();
$THEME->javascripts = array(
    'jquery-1.7.1.min'
);

if ($id) {
    $cm         = get_coursemodule_from_id('clickmeeting', $id, 0, false, MUST_EXIST);
    $course     = $DB->get_record('course', array('id' => $cm->course), '*', MUST_EXIST);
    $clickmeeting  = $DB->get_record('clickmeeting', array('id' => $cm->instance), '*', MUST_EXIST);
    $clickmeeting_conference  = $DB->get_record('clickmeeting_conferences', array('clickmeeting_id' => $clickmeeting->id), '*', MUST_EXIST);
} elseif ($n) {
    $clickmeeting  = $DB->get_record('clickmeeting', array('id' => $n), '*', MUST_EXIST);
    $clickmeeting_conference  = $DB->get_record('clickmeeting_conferences', array('clickmeeting_id' => $clickmeeting->id), '*', MUST_EXIST);
    $course     = $DB->get_record('course', array('id' => $clickmeeting->course), '*', MUST_EXIST);
    $cm         = get_coursemodule_from_instance('clickmeeting', $clickmeeting->id, $course->id, false, MUST_EXIST);
} else {
    error('You must specify a course_module ID or an instance ID');
}

require_login($course, true, $cm);
$context = context_module::instance($cm->id);

/// Print the page header

$PAGE->set_url('/mod/clickmeeting/view.php', array('id' => $cm->id));
$PAGE->set_title(format_string($clickmeeting->name));
$PAGE->set_heading(format_string($course->fullname));
$PAGE->set_context($context);

// other things you may want to set - remove if not needed
//$PAGE->set_cacheable(false);
//$PAGE->set_focuscontrol('some-html-id');
//$PAGE->add_body_class('clickmeeting-'.$somevar);

//// GENERUJEMY TOKEN DLA USER-a
//$token = $DB->get_field('clickmeeting_tokens', 'token', array('user_id' => $USER->id, 'conference_id' => $clickmeeting_conference->conference_id));
//if (empty($token))
//{
//    $token = clickmeeting_generate_token($clickmeeting_conference->conference_id);
//    // wstawiamy wpis do bazy o utworzonym spotkaniu
//    $token_info = new stdClass();
//    $token_info->user_id = $USER->id;
//    $token_info->role = 0;
//    $token_info->conference_id = $clickmeeting_conference->conference_id;
//    $token_info->token = $token;
//
//    $DB->insert_record('clickmeeting_tokens', $token_info);
//}

?>

<?php
    $PAGE->requires->css('/mod/clickmeeting/styles/clickmeeting.css');
    $PAGE->requires->css('/mod/clickmeeting/styles/jquery.popupLight.css');
?>
<script type="text/javascript" src="<?php echo $CFG->wwwroot ?>/mod/clickmeeting/scripts/jquery.min.js"></script>
<script>
    window.cQuery = window.jQuery;
</script>
<script type="text/javascript" src="<?php echo $CFG->wwwroot ?>/mod/clickmeeting/scripts/swfobject.js"></script>
<script type="text/javascript" src="<?php echo $CFG->wwwroot ?>/mod/clickmeeting/scripts/popup.js"></script>
<script type="text/javascript" src="<?php echo $CFG->wwwroot ?>/mod/clickmeeting/scripts/jquery.popupLight.js"></script>
<script type="text/javascript">
    (function ($) {
        $(document).ready(function(){

            $('.enter_meeting').bind('click', function(e){
                e.preventDefault();

                var url = this.href;
                var popup = $.ajaxToPopup({ });

                popup.doRedirect({
                    url: url
                });

                e.stopPropagation();
            });

            $('#tester').bind('click', function(e){
                e.stopPropagation();
                goToUrl({
                    tagName : 'A',
                    href : 'http://google.pl'
                })
                e.preventDefault();
            });

            var embed = $('#copyEmbedRoom');
            if (embed.length && $('#copyToClipboard').length){
                var valueInp = '<script src="<?php echo $clickmeeting_conference->embed_room; ?>"><\/script>';
                embed.val(valueInp);
                var flashvars = {
                    value: valueInp,
                    text: "<?php echo get_string('view:copy_to_clipboard', 'clickmeeting'); ?>"
                };

                var params = {
                    menu: "false",
                    allowFullScreen:"false",
                    allowScriptAccess:"always",
                    wmode:"transparent"
                };

                var attributes = {
                    id: "clipboard",
                    name: "clipboard"
                };

                swfobject.embedSWF("<?php echo $CFG->wwwroot ?>/mod/clickmeeting/extra/copy.swf", "copyToClipboard", "60", "22", "9.0.0", null, flashvars, params, attributes);
            }
        });
    })(window.cQuery);
</script>


<?php

$context = context_module::instance($cm->id);
if (is_siteadmin()) {
    $role = 'host'; // host
} else if(has_capability('mod/clickmeeting:host', $context)) {
//if(has_capability('mod/clickmeeting:host', $context)) {
    $role = 'host';
} else if(has_capability('mod/clickmeeting:presenter', $context)) {
    $role = 'presenter';
} else if (has_capability('mod/clickmeeting:listener', $context)) {
    $role = 'listener';
}

$login_hash = clickmeeting_get_login_url($clickmeeting_conference->conference_id, $USER->email, $USER->firstname . ' ' . $USER->lastname, $role, $clickmeeting_conference->password);

// Output starts here
echo $OUTPUT->header();


echo '<b style="font-size: 110%;">'.get_string('view:room_name', 'clickmeeting').': </b>'.$clickmeeting->name;
echo '<br /><br />';

?>

<div style="float: right; margin-right: 50px;">
    <a href="https://panel.clickmeeting.com/popup_tester.html" onclick="return $(this).popupLight({ iframe:true, width:585, height:460, processingTime:300});"><img src="<?php echo $CFG->wwwroot ?>/mod/clickmeeting/images/baner_en.png" alt="<?php echo get_string('view:test_connection', 'clickmeeting'); ?>" /></a>
</div>

<?php

echo '<b style="font-size: 110%;">'.get_string('view:room_date', 'clickmeeting').': </b>'.$clickmeeting->start_time.' - '.date('Y-m-d H:i:s', strtotime('+'.$clickmeeting->duration.' hours', $clickmeeting->timestart));
echo '<br /><br />';

echo '<b style="font-size: 110%;">'.get_string('view:room_description', 'clickmeeting').': </b>'.$clickmeeting->description;
echo '<br /><br />';

echo '<b style="font-size: 110%;">'.get_string('view:password', 'clickmeeting').': </b>'.$clickmeeting_conference->password;
echo '<br /><br />';

echo '<div style="text-align: center;" class="bt-green"><a class="enter_meeting" href="'.$clickmeeting_conference->room_url.'?l='.$login_hash.'" target="_blank">'.get_string('view:joinmeeting', 'clickmeeting').'</a></div>';
?>


<br /><br />
<?php if ('host' == $role){
    echo '<b style="font-size: 110%;">'.get_string('view:embed', 'clickmeeting').': </b>';
 ?>

<div>
    <div class="formElem">
        <input type="text" style="width: 500px;" readonly="readonly" id="copyEmbedRoom" />
    </div>
    <div class="aft-Inp">
        <div id="copyToClipboard"></div>
    </div>
</div>

<br /><br /><br /><br />


<?php
}

// Finish the page
echo $OUTPUT->footer();
